import codecs
from .codec_base import MSXCodecBase

# Define data first
# ASCII (0x00-0x7F) range mapping (some control characters are allowed)
MSX_ASCII_MAP = {
    i: chr(i) for i in range(0x80)
    if chr(i).isprintable() or chr(i) in '\x00\x07\x08\x09\x0A\x0D\x1B\x20'
}

# MSX Brazilian
MSX_CUSTOM_MAP_BR = {
    0x80: 'Ç', 0x81: 'ü', 0x82: 'é', 0x83: 'â',
    0x84: 'Á', 0x85: 'à', 0x86: '¨', 0x87: 'ç',
    0x88: 'ê', 0x89: 'Í', 0x8A: 'Ó', 0x8B: 'Ú',
    0x8C: 'Â', 0x8D: 'Ê', 0x8E: 'Ô', 0x8F: 'À',
    0x90: 'É', 0x91: 'æ', 0x92: 'Æ', 0x93: 'ô',
    0x94: 'ö', 0x95: 'ò', 0x96: 'û', 0x97: 'ù',
    0x98: 'ÿ', 0x99: 'Ö', 0x9A: 'Ü', 0x9B: '¢',
    0x9C: '£', 0x9D: '¥', 0x9E: 'Cz', 0x9F: 'ƒ',

    0xA0: 'á', 0xA1: 'í', 0xA2: 'ó', 0xA3: 'ú',
    0xA4: 'ñ', 0xA5: 'Ñ', 0xA6: 'ª', 0xA7: 'º',
    0xA8: '¿', 0xA9: '⌐', 0xAA: '¬', 0xAB: '½',
    0xAC: '¼', 0xAD: '¡', 0xAE: '«', 0xAF: '»',
    0xB0: 'Ã', 0xB1: 'ã', 0xB2: 'Ĩ', 0xB3: 'ĩ',
    0xB4: 'Õ', 0xB5: 'õ', 0xB6: 'Ű', 0xB7: 'ű',
    0xB8: 'Ĳ', 0xB9: 'ĳ', 0xBA: '¾', 0xBB: '∽',
    0xBC: '◊', 0xBD: '‰', 0xBE: '¶', 0xBF: '§',
														
    0xC0: '▂', 0xC1: '▚', 0xC2: '▆', 0xC3: '🮂',
    0xC4: '▬', 0xC5: '🮅', 0xC6: '▎', 0xC7: '▞',
    0xC8: '▊', 0xC9: '🮇', 0xCA: '🮊', 0xCB: '🮙',
    0xCC: '🮘', 0xCD: '🭭', 0xCE: '🭯', 0xCF: '🭬',																    
    0xD0: '🭮', 0xD1: '🮚', 0xD2: '🮛', 0xD3: '▘',
    0xD4: '▗', 0xD5: '▝', 0xD6: '▖', 0xD7: '🮖',
    0xD8: 'Δ', 0xD9: '‡', 0xDA: 'ω', 0xDB: '█',
    0xDC: '▄', 0xDD: '▌', 0xDE: '▐', 0xDF: '▀',

    0xE0: 'α', 0xE1: 'ß', 0xE2: 'Γ', 0xE3: 'π',
    0xE4: 'Σ', 0xE5: 'σ', 0xE6: 'µ', 0xE7: 'τ',
    0xE8: 'Φ', 0xE9: 'Θ', 0xEA: 'Ω', 0xEB: 'δ',
    0xEC: '∞', 0xED: '⌀', 0xEE: '∈', 0xEF: '∩',
    0xF0: '≡', 0xF1: '±', 0xF2: '≥', 0xF3: '≤',
    0xF4: '⌠', 0xF5: '⌡', 0xF6: '÷', 0xF7: '≈',
    0xF8: '°', 0xF9: '∙', 0xFA: '·', 0xFB: '√',
    0xFC: 'ⁿ', 0xFD: '²', 0xFE: '■'
}

# Combined map (MSX code to character)
MSX_CHAR_MAP_BR = dict(MSX_ASCII_MAP)
MSX_CHAR_MAP_BR.update(MSX_CUSTOM_MAP_BR)

# Reverse lookup (character to MSX code)
MSX_CHAR_REVERSE_MAP_BR = {v: k for k, v in MSX_CHAR_MAP_BR.items()}

# Graphic character mapping (2-byte characters)
GRAPHIC_CHAR_MAP_BR = {
    '☺': (0x01, 0x41), '☻': (0x01, 0x42), '♥': (0x01, 0x43),
    '♦': (0x01, 0x44), '♣': (0x01, 0x45), '♠': (0x01, 0x46),
    '•': (0x01, 0x47), '◘': (0x01, 0x48), '○': (0x01, 0x49),
    '◙': (0x01, 0x4A), '♂': (0x01, 0x4B), '♀': (0x01, 0x4C),
    '♪': (0x01, 0x4D), '♫': (0x01, 0x4E), '☼': (0x01, 0x4F),
    '⟊': (0x01, 0x50), '┴': (0x01, 0x51), '┬': (0x01, 0x52),
    '┤': (0x01, 0x53), '├': (0x01, 0x54), '┼': (0x01, 0x55),
    '│': (0x01, 0x56), '─': (0x01, 0x57), '┌': (0x01, 0x58),
    '┐': (0x01, 0x59), '└': (0x01, 0x5A), '┘': (0x01, 0x5B),
    '╳': (0x01, 0x5C), '╱': (0x01, 0x5D), '╲': (0x01, 0x5E),
    '🮯': (0x01, 0x5F)
}

# Reverse lookup for graphic characters
GRAPHIC_CHAR_REVERSE_MAP_BR = {v: k for k, v in GRAPHIC_CHAR_MAP_BR.items()}


class MSXBrazilianCodec(MSXCodecBase):
    """Brazilian MSX character set codec"""
    
    def __init__(self):
        super().__init__('msx-br', MSX_CUSTOM_MAP_BR, GRAPHIC_CHAR_MAP_BR)


# Create codec instance
msx_codec_br = MSXBrazilianCodec().codec_info
